      subroutine lll601( result )

C  Calculate the matrix element in the LLL (Limited Leading-Log) 
C  approximation.
C
C  Process 601 is "u g -> a a u" or "dbar g -> a a dbar".
C  They can be distinguished by looking at the initial state "kfcode".
C  The branch mode in the collinear approx. is unique: g -> qbar q 
C  (q = d or u) in the initial state.
C  Note that "u(d)" denotes a generic "u(d)-type" quark.
C  The "kfcode" may not strictly correspond to this event since it 
C  corresponds to the particle configuration of the event to generate.

      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'

      parameter ( nextrn= 5, nextrn0= nextrn -1 )
      integer kfcode1(nextrn),kcharg1(nextrn),kf0(nextrn0)
      real*8 bdir(3),ptemp(4),p0(4,nextrn0)
      parameter ( Tr= 0.5d0 )

      result= 0.d0

C  Save the particle information of the hard interaction, because 
C  it is replaced in the calculation of non-radiative process.
C  It would be enough to save/restore "kfcode" since "amass1/2" are 
C  recalculated when the LHA common is filled.
      do i=1,nextrn
         kfcode1(i)= kfcode(i)
         kcharg1(i)= kcharg(i)
      enddo

C  Squared cm energy of this event.
C  The input event must be in the cm frame with initial-state particles 
C  aligned along the z axis.
      stot= ( pgrc(4,1) +pgrc(4,2) )**2

C  === Initial-state radiation ===

C  The branch is unique.
      irad= 5
      imoth= 2

C  Momentum transfer.
      do i=1,4
         ptemp(i)= pgrc(i,imoth) -pgrc(i,irad)
      enddo
      qsq= ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 -ptemp(4)**2

C  Boost to the cm frame of the non-radiation subsystem.
      call bstcm( pgrc(1,1), ptemp,
     >     pcm, ecm1, ecm2, bdir, gamma, phi, theta )

C  Squared cm energy of the non-radiative subsystem.
C  The branch parameter is defined by its fraction.
      s0= ( ecm1 +ecm2 )**2
      zbr= s0/stot

C  The initial-state particles must be massless in this approximation.
      pini= sqrt( s0 )/2.d0
C  Set the four-momenta in the subsystem's cm frame.
      p0(1,1)= 0.d0
      p0(2,1)= 0.d0
      p0(3,1)= pini
      p0(4,1)= pini
      p0(1,2)= 0.d0
      p0(2,2)= 0.d0
      p0(3,2)= -pini
      p0(4,2)= pini

C  Boost the final-state particles to the CM frame.
      j= 2
      do i=3,nextrn
         if( i.ne.irad ) then
            j= j +1
            call bstfw( pgrc(1,i), bdir, gamma, phi, theta, p0(1,j) )
         endif
      enddo

C  No approximation if the radiation is harder than the factorization scale.
      call setscale( 600, p0, qren, qfac )
      if( qsq.lt.qfac**2 ) then

C  Matrix element of the non-radiative process.
         kf0(1)= min( kfcode1(1), kfcode1(2) )
         kf0(2)= -kf0(1)
         kf0(3)= 22
         kf0(4)= 22
         call me600( kf0, p0, result0 )

C  Matrix element in the factorized collinear approximation 
         factor= 8*pi*alphas*Tr*( zbr**2 +( 1 -zbr )**2 )/zbr/qsq
         result= result +result0*factor

C  Restore the saved variables.
         do i=1,nextrn
            kfcode(i)= kfcode1(i)
            kcharg(i)= kcharg1(i)
         enddo

      endif

C  === Final-state radiation ===
C  QED: q(qbar) -> q(qbar) + gamma

C  The quark in the final state is considered to be the child.
      ichild= 5
      if( mod( iabs( kfcode(ichild) ), 2 ).eq.1 ) then
         cqed= 1.d0/9.d0
      else
         cqed= 4.d0/9.d0
      endif

      pini= sqrt( stot )/2.d0

C  Both gammas can be radiation.
      do irad=3,4

C  Momentum sum.
         do i=1,4
            ptemp(i)= pgrc(i,ichild) +pgrc(i,irad)
         enddo

C  Parameters of the considered branch.
C  "zbr" is defined to be the momentum fraction in an infinite-momentum frame.
         psum= sqrt( ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 )
         esum= ptemp(4)
         qsq= esum**2 -psum**2
         pl= ( pgrc(1,ichild)*ptemp(1) +pgrc(2,ichild)*ptemp(2)
     >        +pgrc(3,ichild)*ptemp(3) )/psum
         zbr= ( pl*( esum +psum ) +qsq/2. )
     >        /( psum*( esum +psum ) +qsq )

C  Requrement of a forward radiation because QCDPSf does not emit 
C  backward radiations.
         if( pl.gt.0.d0 .and. pl.lt.psum ) then

C  Set the four-momenta in the subsystem's cm frame.
C  The momentum adjustment is not applied to the final state.
C  Instead, the initial-state momenta are changed to conserve the energy.
            p0(1,1)= 0.d0
            p0(2,1)= 0.d0
            p0(3,1)= psum
            p0(4,1)= psum
            p0(1,2)= 0.d0
            p0(2,2)= 0.d0
            p0(3,2)= -psum
            p0(4,2)= psum

            if( irad.eq.3 ) then
               igam= 4
            else
               igam= 3
            endif
            p0(1,3)= pgrc(1,igam)
            p0(2,3)= pgrc(2,igam)
            p0(3,3)= pgrc(3,igam)
            p0(4,3)= pgrc(4,igam)
            p0(1,4)= ptemp(1)
            p0(2,4)= ptemp(2)
            p0(3,4)= ptemp(3)
            p0(4,4)= psum

C  No approximation if the radiation is harder than the factorization scale.
            call scale_fsub( 610, p0, qmax )
            if( qsq.lt.qmax**2 ) then

C  Matrix element of the non-radiative process.
               kf0(1)= kfcode1(1)
               kf0(2)= kfcode1(2)
               kf0(3)= 22
               kf0(4)= kfcode1(5)
               call me610( kf0, p0, result0 )

C  Radiation factor.
               factor= 8.d0*pi*alpha*cqed*( 1 +zbr**2 )/( 1 -zbr )/qsq

C  s-hat correction.
               s0= ( 2.d0*psum )**2
               factor= factor * stot/s0

C  Matrix element in the factorized collinear approximation.
               result= result +result0*factor

C  Restore the saved variables.
               do i=1,nextrn
                  kfcode(i)= kfcode1(i)
                  kcharg(i)= kcharg1(i)
               enddo

            endif
         endif

      enddo

      return
      end

      subroutine mzero601
      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'
      common/msave/psave(4,10),am1save(10),am2save(10)
C  Save the original information.
      do i=1,5
         do j=1,4
            psave(j,i)= pgrc(j,i)
         enddo
         am1save(i)= amass1(i)
         am2save(i)= amass2(i)
      enddo
C  For the process 601, the radiation (quark) may be massive.
      pgrc(4,5)= sqrt( pgrc(1,5)**2 +pgrc(2,5)**2 +pgrc(3,5)**2 )
C  Modification is only for the initial-state partons.
      ecm= pgrc(4,3) +pgrc(4,4) +pgrc(4,5)
      pini= ecm/2.d0
      pgrc(1,1)= 0.d0
      pgrc(2,1)= 0.d0
      pgrc(3,1)= pini
      pgrc(4,1)= pini
      pgrc(1,2)= 0.d0
      pgrc(2,2)= 0.d0
      pgrc(3,2)= -pini
      pgrc(4,2)= pini
      amass1(1)= 0.d0
      amass2(1)= 0.d0
      amass1(5)= 0.d0
      amass2(5)= 0.d0
      return
      end

      subroutine mrest601
      implicit real*8(a-h,o-z)
      include 'inclk.h'
      include 'grchad.inc'
      common/msave/psave(4,10),am1save(10),am2save(10)
      do i=1,5
         do j=1,4
            pgrc(j,i)= psave(j,i)
         enddo
         amass1(i)= am1save(i)
         amass2(i)= am2save(i)
      enddo
      return
      end
