      SUBROUTINE QCDPSb(Ipro,Ipar,Ib,Escl,Xin)
C Initial-state "backward" PS evolution based on the algorithm in
C the subroutine "pysspa" of PYTHIA.
C Ref: Bengtsson, Sjostrand, and Zijl, Z. Phys. C 32, 67 (1986)
C
C Input
C    Ipro: beam particle (+1: protpn, -1: anti-protpn )
C    Ipar: PDG code of the parton (1,2,3,4,5,21 for d,u,s,c,b,g)
C    Ib: beam diretion (+1: +Z direction, -1: -Z direction)
C    Escl: energy scale of the PS evolution (maximum virtuality)
C    Xin: "x" value of the parton
C
C Output
C    Parameters in the arrays, PLPTN and NLPTN.
C    The definition of the contents is the same as that in QCDPS,
C    so that PSKINI can be directly applied to them.
C
C Internal convention:
C    The beam hadron is always assumed to be a proton.
C    Each iteration step is a process to determine the mother parton "a" 
C    corresponding to an evolving parton "b" in the branch, a -> b c, 
C    where "c" is a timelike radiation.
C    The particle code is 0 for gluon, and the codes for quarks are 
C    the same as the PDG ones; i.e., 0,1,2,3,4,5 for g,d,u,s,c,b.
C    The array IDvonv2 converts the PDG codes to another (original QCDPS) 
C    convention, where the codes for d and u are interchanged and that 
C    for the gluon is 0.
C    The branch information are stored at the end of the storage arrays, 
C    PLPTN and NLPTN, during the evolution.
C    The particle codes are converted to the PDG ones when stored.
C    The parameters are copied to the ordinary positions after the evolution 
C    is completed.
C    The particle codes for quarks are converted to those for their 
C    anti-particles if the beam is the anti-proton.
C
C Aug. 5, 2009; S. Odaka
C
      IMPLICIT REAL*8(A-H,O-Z)
      include 'qpincl.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      parameter ( nmax= 10000 )
      common/QPinfo/Qps,ipps(3,2)
      real*8 xpdf(-6:6),wapp(-6:6)

      Es2= Escl**2
      Ibeam= Ib
      Iproton= Ipro

C  Check the particle ID
      if( Ipar.lt.-5 .or. Ipar.gt.21 ) then
         write(6,*)'QCDPSb: wrong particle ID ',Ipar
         stop
      endif
C  Partons are considered to be their anti-partner when the beam is 
C  the anti-proton.
      Iparton= Ipar
      if( Ipar.ne.21 ) Iparton= Iproton*Ipar
      Ifo= IDconv2(Iparton)
      if( Ifo.eq.999 ) then
         write(6,*)'QCDPSb: wrong particle ID ',Ipar
         stop
      endif

C  Initial condition.
C  Hereafter, id = 0 for gluons.
      tevol= log( Es2/QCDLd**2 )
      if( Iparton.eq.21 ) then
         idb= 0
      else
         idb= Iparton
      endif
      xb= Xin

      nbranch= 0
      ipback= nmax +1

C  Do not evolve if the initial energy scale is already small.
      if( Es2.lt.Q2PDF(iabs( Ifo )) ) goto 9

C -- Top of the iteration.
 1    continue

C  Terminate the evolution if the momentum fraction becomes too large.
      if( xb.ge.( 1.d0 -eps ) ) goto 9

C  PDF for the current condition.
      scale= QCDLd*exp( tevol/2.d0 )
      call grcpdf( xb, scale, xpdf )
      if( xpdf(idb).lt.1.d-20 ) goto 9

C  Approximation of the splitting-function integral.
      do i=-6,6
         wapp(i)= 0.d0
      enddo
C  'q' -> 'q' g, 'g' -> 'q' qbar
      if( idb.ne.0 ) then
         wapp(idb)= 2.d0*Cf*log( ( 1.d0 -xb )/eps )
         wapp(0)= 0.5d0*( 1.d0 -eps -xb )
C  'q' -> 'g' q, 'g' -> 'g' g
      else
         approx= 4.d0*Cf*( 1.d0/sqrt( xb ) -1.d0/sqrt( 1.d0 -eps ) )
         do i=-5,5
            if( i.ne.0 ) then
               wapp(i)= approx
            endif
         enddo
         wapp(0)= Cg*log( ( 1.d0 -eps )*( 1.d0 -xb )/eps/xb )
      endif

C  Apply the PDF factor (approximation).
      wapps= 0.d0
      do i=-5,5
         wapp(i)= wapp(i)*xpdf(i)/xpdf(idb)
         if( wapp(i).lt.0.d0 ) wapp(i)= 0.d0
         wapps= wapps +wapp(i)
      enddo
      if( wapps.lt.1.d-20 ) goto 9

C  Determination of the next branch.
      ntry= 0
 2    continue
      if( ntry.ge.500 ) then
         write(6,*)'QCDPSb: cannot find a new branch'
         goto 9
C         stop
      endif
      ntry= ntry +1

C  New t value.
      tnew= tevol*exp( log( QPRAND(Idum) )*beta/2.d0/wapps )

C  The parent parton.
      wrand= QPRAND(Idum)*wapps
      ida= -5
 3    wrand= wrand -wapp(ida)
      if( wrand.ge.0.d0 ) then
         ida= ida +1
         if( ida.le.5 ) goto 3
      endif
      if( ida.ge.6 ) ida= idb
C  Protection.
      if( ida.ne.0 .and. idb.ne.0 .and. ida.ne.idb ) then
         write(6,*)'QCDPSb: illegal ID ',ida,' for the child ',idb
         write(6,*)'  wapps ',wapps,', wapp ',wapp
         stop
      endif
      if( xpdf(ida).lt.1.d-20 ) goto 2
      
C  Terminate or not.
      qsq= exp( tnew )*QCDLd**2
      id= ida
      if( id.eq.0 ) id= 21
      if( qsq.lt.Q2PDF(iabs( IDconv2(id) )) ) goto 9
      tevol= tnew

C  z parameter, and the weight for the rejection.
C  'q' -> 'q' g, 'g' -> 'q' qbar
      if( idb.ne.0 ) then
         if( ida.ne.0 ) then
            zbr= 1.d0 -( 1.d0 -xb )*( eps/( 1.d0 -xb ) )**QPRAND(Idum)
            wgt= 0.5d0*( 1.d0 +zbr**2 )
         else
            zbr= xb +( 1.d0 -eps -xb )*QPRAND(Idum)
            wgt= 1.d0 -2.d0*zbr*( 1.d0 -zbr )
         endif
C  'q' -> 'g' q, 'g' -> 'g' g
      else
         if( ida.ne.0 ) then
            zbr= xb/( 1.d0 
     >           -( 1.d0 -sqrt( xb/( 1.d0 -eps) ) )*QPRAND(Idum) )**2
            wgt= 0.5d0*( 1.d0 +( 1.d0 -zbr )**2 )*sqrt( zbr )
         else
            zbr= 1.d0/( 1.d0 +( 1.d0 -xb )/xb
     >           *( eps*xb/( 1.d0 -eps )/( 1.d0 -xb ) )**QPRAND(Idum) )
            wgt= ( 1.d0 -zbr*( 1.d0 -zbr ) )**2
         endif
      endif
      xa= xb/zbr
      if( xa.gt.1.d0 ) then
         xa= 1.d0
         zbr= xb/xa
      endif

C  Rejection.
C  PDF at the new energy scale.
      scale= QCDLd*exp( tevol/2.d0 )
      call grcpdf1( idb, xb, scale, xpdfb )
      call grcpdf1( ida, xa, scale, xpdfa )
      if( xpdfb.lt.1.d-20 ) goto 9
C  Total weight = exact/approximation.
      wgt= wgt*( xpdfa/xpdfb )/( xpdf(ida)/xpdf(idb) )
CCC      if( wgt.gt.1.d0 ) goto 2
      if( QPRAND(Idum).gt.wgt ) goto 2

      nbranch= nbranch +1

C  Filling.
      if( ida.eq.idb ) then
         Idiag= 1
      else
         Idiag= 0
      endif

C  radiation
      ipback= ipback -1
      do i=1,10
         PLPTN(i,ipback)= 0.d0
         NLPTN(i,ipback)= 0
      enddo

      PLPTN(1,ipback)= 1.d0 -zbr
      PLPTN(2,ipback)= 0.d0
      PLPTN(10,ipback)= 1.d0

      id= ida -idb
      if( id.eq.0 ) id= 21

      NLPTN(1,ipback)= id
      NLPTN(2,ipback)= 3
      NLPTN(3,ipback)= 0
      NLPTN(4,ipback)= 0
      NLPTN(5,ipback)= 2
      NLPTN(6,ipback)= 1
      NLPTN(8,ipback)= Idiag

C  evolved.
      ipback= ipback -1
      do i=1,10
         PLPTN(i,ipback)= 0.d0
         NLPTN(i,ipback)= 0
      enddo
      qsq= exp( tevol )*QCDLd**2

      PLPTN(1,ipback) = zbr
      PLPTN(2,ipback) = -qsq
      PLPTN(10,ipback) = 1.d0

      id= idb
      if( id.eq.0 ) id= 21

      NLPTN(1,ipback) = id
      NLPTN(2,ipback) = 2
      if( nbranch.eq.1 ) then
         NLPTN(3,ipback)= 0
         NLPTN(4,ipback)= 0
      else
         NLPTN(3,ipback)= 2
         NLPTN(4,ipback)= 2
      endif
      NLPTN(5,ipback) = 2
      NLPTN(6,ipback) = -1
      NLPTN(8,ipback) = Idiag

C  Go to the next branch
      idb= ida
      xb= xa
      goto 1

C -- The evolution is completed.
 9    continue
      npre= NTOP

C  proton/anti-proton
      NTOP= NTOP +1

      PLPTN(1,NTOP)= 1.d0
      PLPTN(2,NTOP)= 0.93827d0**2
      PLPTN(3,NTOP)= 0.d0
      PLPTN(4,NTOP)= 0.d0
      PLPTN(5,NTOP)= 0.d0
      PLPTN(6,NTOP)= Ibeam*sqrt( Ebeam**2 -PLPTN(2,NTOP) )
      PLPTN(7,NTOP)= Ebeam
      PLPTN(10,NTOP)= 1.d0

      NLPTN(1,NTOP)= Iproton*2212
      NLPTN(2,NTOP)= 0
      NLPTN(3,NTOP)= 1
      NLPTN(4,NTOP)= 1
      NLPTN(5,NTOP)= 3
      NLPTN(6,NTOP)= 1

C  remaining parton
      NTOP= NTOP +1
      id= idb
      if( id.eq.0 ) id= 21
      qsq= exp( tevol )*QCDLd**2

      PLPTN(1,NTOP)= xb
      PLPTN(2,NTOP)= -qsq
      PLPTN(3,NTOP)= 0.d0
      PLPTN(4,NTOP)= 0.d0
      PLPTN(5,NTOP)= 0.d0
      PLPTN(6,NTOP)= Ibeam*Ebeam*xb
      PLPTN(7,NTOP)= Ebeam
      PLPTN(10,NTOP)= 1.d0

      if( id.eq.21 ) then
         NLPTN(1,NTOP)= id
      else
         NLPTN(1,NTOP)= Iproton*id
      endif
      NLPTN(2,NTOP)= 1
      NLPTN(3,NTOP)= 0
      NLPTN(4,NTOP)= 0
      NLPTN(5,NTOP)= 2
      NLPTN(6,NTOP)= -1

C  Copy the branch information.
      if( nbranch.gt.0 ) then
         NLPTN(3,NTOP)= 2
         NLPTN(4,NTOP)= 1
         NLPTN(2,ipback)= 1
         NLPTN(2,ipback+1)= 2

         do i=ipback,nmax
            NTOP= NTOP +1
            do j=1,10
               PLPTN(j,NTOP)= PLPTN(j,i)
               NLPTN(j,NTOP)= NLPTN(j,i)
            enddo
            if( NLPTN(1,NTOP).ne.21 ) then
               NLPTN(1,NTOP)= Iproton*NLPTN(1,NTOP)
            endif
         enddo
      endif

C  Set the pointers.
      if( Ibeam.gt.0 ) then
         i= 1
      else
         i= 2
      endif
      ipfin= NTOP -1
      if( ( NTOP -npre ).lt.4 ) ipfin= NTOP
      ipps(1,i)= npre +1
      ipps(2,i)= ipfin
      ipps(3,i)= NTOP

      return

      end
