C  Routines for the correction to be applied to PS-applied events, 
C  that compensates for the alteration in the divergence subtraction 
C  due to the presence of the "soft-gluon divergence" in multi-jet 
C  production processes.
C  The interface to process-dependent routines is to be inserted 
C  during the configuration stage of the installation.
C  The statements to be inserted have to be written in the files 
C  named "sgcchk.ins" and "sgcval.ins" in each process directory.
C  Aug. 4, 2014, S.O.
C--------+---------+---------+---------+---------+---------+---------+
      subroutine sgcchk( igsub, ppart, idpart, iret )
C  Check whether the soft-gluon correction should be applied to the given 
C  event or not.
C  iret = 0: Yes, 1: No
      implicit real*8(a-h,o-z)
      real*8 ppart(4,10)
      integer idpart(10)

      iret= 1

C Add process-dependent interface statements. (SGCCHK)

      return
      end
C--------+---------+---------+---------+---------+---------+---------+
      subroutine sgcval( igsub, ppart, idpart, prad, weight )
C  Evaluate the soft-divergence correction factor to the given event.
      implicit real*8(a-h,o-z)
      real*8 ppart(4,10),prad(4)
      integer idpart(10)

      if( igsub.lt.100 ) then
         print *,'sgcval: illegal process number: ',
     >        'igsub = ',igsub
         stop
C Add process-dependent interface statements. (SGCVAL)
      else
         print *,'sgcval: correction requested but not supported for ',
     >        'igsub = ',igsub
         stop
      endif

      return
      end
C--------+---------+---------+---------+---------+---------+---------+
      subroutine sgcorr( igsub, weight )
C  Evaluate the correction to PS-applied events that arises from the existence
C  of additional soft-gluon divergence; effective only for processes having
C  three or more external colored particles including initial state ones.
C
C  We pick up the hardest radiation from the resultant PS.
C  The correction is evaluated on the basis of the "combined approximation" 
C  technique used in the LLL subtraction when the hardest radiation is 
C  a gluon;
C       weight = combined approx./collinear approx. for PS,
C  where  
C       combined = ( 1 -rsoft )*collinear + rsoft*soft.
C
C  29 Jan. 2014, S.Odaka
      implicit real*8(a-h,o-z)
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP
      common/QPinfo/Qps,ipps(3,2)

      real*8 ppart(4,10),prad(4),pmoth(4),pchild(4)
      integer ippart(10),idpart(10),ipstack(100)
      real*8 ptemp(4),bdir(3)

      weight= 1.d0

C  Check the process-dependent request.
C  Nothing is done if not requested.
      call sgcreq( igsub, ireq )
      if( ireq.eq.0 ) return

C  Pick up the particle info of the non-radiative process.
C  initial state
      if( PLPTN(6,ipps(2,1)).ge.PLPTN(6,ipps(2,2)) ) then
         ippart(1)= ipps(2,1)
         ippart(2)= ipps(2,2)
      else
         ippart(1)= ipps(2,2)
         ippart(2)= ipps(2,1)
      endif
      do ipart=1,2
         ip= ippart(ipart)
         do i=1,4
            ppart(i,ipart)= PLPTN(i+3,ip)
         enddo
         idpart(ipart)= NLPTN(1,ip)
      enddo
C  final state
      npart= 2
      nnoevol= 0
      do ip=1,NTOP
         if( NLPTN(5,ip).eq.10 .and. NLPTN(3,ip).eq.0 ) then
            nnoevol= nnoevol +1
            npart= npart +1
            ippart(npart)= ip
            idpart(npart)= NLPTN(1,ip)
            do i=1,4
               ppart(i,npart)= PLPTN(i+3,ip)
            enddo
         endif
      enddo
      nevol= 0
      do ip=1,NTOP
         if( NLPTN(5,ip).eq.11 ) then
            nevol= nevol +1
            npart= npart +1
            ippart(npart)= ip
            idpart(npart)= NLPTN(1,ip)
            do i=1,4
               ppart(i,npart)= PLPTN(i+3,ip)
            enddo
         endif
      enddo

C  Find the hardest radiation.
C  - Initial-state PS -
C  The search is done for main branches only.
      iphardest= 0
      pthardest= 0.d0
      do ipart=1,2
         ipchild= ippart(ipart)
 1       continue
         irmoth= NLPTN(2,ipchild)
         if( irmoth.lt.1 ) goto 2
         ipmoth= ipchild -irmoth
         nchild= NLPTN(3,ipmoth)
         if( nchild.lt.2 ) goto 2
         ip1= ipmoth +NLPTN(4,ipmoth)
         ip2= ip1 +nchild -1
         do ip=ip1,ip2
            if( NLPTN(6,ip).gt.0 ) then
               idrad= NLPTN(1,ip)
               ptbr= sqrt( PLPTN(3,ip) )
               if( ptbr.gt.pthardest ) then
                  iphardest= ip
                  pthardest= ptbr
                  ipartbr= ipart
               endif
            endif
         enddo
         idchild= NLPTN(1,ipchild)
         idmoth= NLPTN(1,ipmoth)
C  Terminate the search if the flavor changed.
         if( idmoth.eq.idchild ) then
            ipchild= ipmoth
            goto 1
         endif
 2       continue
      enddo
C  - Final-state PS -
C  Examine the quark branches only if the jet is initiated by a quark, 
C  while both gluons are examined if initiated by a gluon and branched to gg.
      if( nevol.gt.0 ) then
         do ipart=nnoevol+3,npart
            ipmoth= ippart(ipart)
            nstack= 0
 3          continue
            nchild= NLPTN(3,ipmoth)
            if( nchild.eq.0 ) then
               goto 4
            elseif( nchild.eq.1 ) then
               ipmoth= ipmoth +NLPTN(4,ipmoth)
               goto 3
            elseif( nchild.ne.2 ) then
               print *,'num of children = ',nchild
               stop
            endif
            idmoth= NLPTN(1,ipmoth)
            ipchild= ipmoth +NLPTN(4,ipmoth)
            if( idmoth.eq.21 ) then
               if( PLPTN(1,ipchild).ge.0.5d0 ) then
                  iprad= ipchild +1
               else
                  iprad= ipchild
                  ipchild= iprad +1
               endif
            else
               if( NLPTN(1,ipchild).eq.21 ) then
                  iprad= ipchild
                  ipchild= iprad +1
               else
                  iprad= ipchild +1
               endif
            endif
            ptbr= sqrt( PLPTN(3,iprad) )
            if( ptbr.gt.pthardest ) then
               iphardest= iprad
               pthardest= ptbr
               ipartbr= ipart
            endif
            idchild= NLPTN(1,ipchild)
C  Terminate the search if the flavor changed.
            if( idchild.eq.idmoth ) then
               ipmoth= ipchild
C  Save the pointer to the radiation if a g -> gg branch.
               if( idchild.eq.21 .and. nstack.lt.100 ) then
                  nstack= nstack +1
                  ipstack(nstack)= iprad
               endif
               goto 3
            endif
 4          continue
C  Examine the branches of saved gluon radiations.
            if( nstack.gt.0 ) then
               ipmoth= ipstack(nstack)
               nstack= nstack -1
               goto 3
            endif
         enddo
      endif
      if( iphardest.lt.1 ) return

C  Correction is applied only for gluon radiation.
      iprad= iphardest
      idrad= NLPTN(1,iprad)
      if( idrad.ne.21 ) return

      ipmoth= iprad -NLPTN(2,iprad)
      ipchild= ipmoth +NLPTN(4,ipmoth)
      if( ipchild.eq.iprad ) ipchild= ipchild +1

      do i=1,4
         pchild(i)= PLPTN(i+3,ipchild)
         prad(i)= PLPTN(i+3,iprad)
         pmoth(i)= pchild(i) +prad(i)
      enddo

C  Move to the CM frame.
      call bstcm( ppart(1,1), ppart(1,2),
     >     P1, E1, E2, bdir, gamma, phi, theta )

      pxsum= 0.d0
      pysum= 0.d0
      pzsum= 0.d0
      esum= 0.d0
      do ipart=3,npart
         call bstfw( ppart(1,ipart), bdir, gamma, phi, theta, ptemp )
         do i=1,4
            ppart(i,ipart)= ptemp(i)
         enddo
         pxsum= pxsum +ptemp(1)
         pysum= pysum +ptemp(2)
         pzsum= pzsum +ptemp(3)
         esum= esum +ptemp(4)
      enddo
      ecm= sqrt( esum**2 -pxsum**2 -pysum**2 -pzsum**2 )

      call bstfw( pmoth, bdir, gamma, phi, theta, ptemp )
      do i=1,4
         pmoth(i)= ptemp(i)
      enddo
      call bstfw( pchild, bdir, gamma, phi, theta, ptemp )
      do i=1,4
         pchild(i)= ptemp(i)
      enddo
      call bstfw( prad, bdir, gamma, phi, theta, ptemp )
      do i=1,4
         prad(i)= ptemp(i)
      enddo

C  Fine adjustment.
      phi= 0.d0
      theta= 0.d0
      psum= sqrt( pxsum**2 +pysum**2 +pzsum**2 )
      bdir(1)= pxsum/psum
      bdir(2)= pysum/psum
      bdir(3)= pzsum/psum
      gamma= esum/ecm

      do ipart=3,npart
         call bstfw( ppart(1,ipart), bdir, gamma, phi, theta, ptemp )
         do i=1,4
            ppart(i,ipart)= ptemp(i)
         enddo
      enddo

C  Force all final-state partons be massless.
      ievol= nnoevol+3
      esum= 0.d0
      do ipart=3,npart
         if( ipart.ge.ievol ) then
            ppart(4,ipart)= sqrt( ppart(1,ipart)**2
     >           +ppart(2,ipart)**2 +ppart(3,ipart)**2 )
         endif
         esum= esum +ppart(4,ipart)
      enddo
      shat= esum**2

C  Check the event.
      pini= esum/2.d0
      ppart(1,1)= 0.d0
      ppart(2,1)= 0.d0
      ppart(3,1)= pini
      ppart(4,1)= pini
      ppart(1,2)= 0.d0
      ppart(2,2)= 0.d0
      ppart(3,2)= -pini
      ppart(4,2)= pini
      call sgcchk( igsub, ppart, idpart, iret )
      if( iret.ne.0 ) return

C  PS branch parameters of the hardest branch.
      if( ipartbr.le.2 ) then
         zbr= PLPTN(1,ipchild)
         qsqbr= PLPTN(2,ipchild)
      else
         zbr= PLPTN(1,ipchild)
         qsqbr= PLPTN(2,ipmoth)
      endif
      ptbr= sqrt( PLPTN(3,iprad) )

C  Rotate the branch to align the mother along the corresponding parton 
C  in the non-rad. process.
      do i=1,4
         ptemp(i)= prad(i)
      enddo
      call rotmin( prad, pmoth, ppart(1,ipartbr) )

C  Construct the radiative process.
      qsqbr= abs( qsqbr )
      if( ipartbr.le.2 ) then
         shat1= shat/zbr
         pini1= sqrt( shat1 )/2.d0
         ppart(1,1)= 0.d0
         ppart(2,1)= 0.d0
         ppart(3,1)= pini1
         ppart(4,1)= pini1
         ppart(1,2)= 0.d0
         ppart(2,2)= 0.d0
         ppart(3,2)= -pini1
         ppart(4,2)= pini1
         pradabs= ( 1.d0 -zbr )*pini1
C         ptrad= ptbr
CC         ptrad= sqrt( ( 1.d0 -zbr )*qsqbr )
C         pt2max= ( 1.d0 -zbr )**2*shat/4.d0/zbr
C         if( ptrad**2.gt.pt2max ) ptrad= sqrt( pt2max )
C         pttmp= sqrt( prad(1)**2 +prad(2)**2 )
C         prad(1)= prad(1)*ptrad/pttmp
C         prad(2)= prad(2)*ptrad/pttmp
C         pchild(1)= -prad(1)
C         pchild(2)= -prad(2)
C         plrad= sqrt( pradabs**2 -ptrad**2 )
C  Redefine the Q^2 from pt and z.
C         qsqbr= ptbr**2/( 1.d0 -zbr )
         plrad= pradabs -qsqbr/2.d0/pini1
         pt2rad= pradabs**2 -plrad**2
         if( pt2rad.lt.1.d0 ) return
         ptrad= sqrt( pt2rad )
         pttmp= sqrt( prad(1)**2 +prad(2)**2 )
         prad(1)= prad(1)*ptrad/pttmp
         prad(2)= prad(2)*ptrad/pttmp
         pchild(1)= -prad(1)
         pchild(2)= -prad(2)
         if( ipartbr.eq.1 ) then
            prad(3)= plrad
            pchild(3)= pini1 -plrad
         else
            prad(3)= -plrad
            pchild(3)= -( pini1 -plrad )
         endif
         prad(4)= pradabs
         pchild(4)= pini1 -pradabs
         if( ipartbr.eq.1 ) then
            call bstcm( pchild, ppart(1,2),
     >           pcm, ecm1, ecm2, bdir, gamma, phi, theta )
         else
            call bstcm( ppart(1,1), pchild,
     >           pcm, ecm1, ecm2, bdir, gamma, phi, theta )
         endif
         do ipart=3,npart
            call bstbk( ppart(1,ipart), bdir, gamma, phi, theta, ptemp )
            do i=1,4
               ppart(i,ipart)= ptemp(i)
            enddo
         enddo
      else
         ptrans= ppart(1,ipartbr)**2 +ppart(2,ipartbr)**2
         pabs= sqrt( ptrans +ppart(3,ipartbr)**2 )
         ptrans= sqrt( ptrans )
         if( ptrans.lt.1.d-12 ) then
            print '('' sgcorr: FSR emitter pt is too small.'')'
            stop
         endif
         csph= ppart(1,ipartbr)/ptrans
         snph= ppart(2,ipartbr)/ptrans
         csth= ppart(3,ipartbr)/pabs
         snth= sqrt( 1.d0 -csth**2 )
         ptemp(1)= csth*csph*prad(1) +csth*snph*prad(2) -snth*prad(3)
         ptemp(2)= -snph*prad(1) +csph*prad(2)
         ptemp(3)= snth*csph*prad(1) +snth*snph*prad(2) +csth*prad(3)
         pttmp= sqrt( ptemp(1)**2 +ptemp(2)**2 )
         ptemp(1)= ptemp(1)*ptbr/pttmp
         ptemp(2)= ptemp(2)*ptbr/pttmp
         pcone= ( pabs +sqrt( pabs**2 +qsqbr ) )/2.d0
         zrad= 1.d0 -zbr
         ptemp(3)= zrad*pcone -ptbr**2/4.d0/zrad/pcone
         prad(1)= csth*csph*ptemp(1) -snph*ptemp(2) +snth*csph*ptemp(3)
         prad(2)= csth*snph*ptemp(1) +csph*ptemp(2) +snth*snph*ptemp(3)
         prad(3)= -snth*ptemp(1) +csth*ptemp(3)
         prad(4)= sqrt( prad(1)**2 +prad(2)**2 +prad(3)**2 )
         do i=1,3
            ppart(i,ipartbr)= ppart(i,ipartbr) -prad(i)
         enddo
         ppart(4,ipartbr)= sqrt( ppart(1,ipartbr)**2
     >        +ppart(2,ipartbr)**2 +ppart(3,ipartbr)**2 )
         do i=1,4
            ptemp(i)= 0.d0
            do ipart=3,npart
               ptemp(i)= ptemp(i) +ppart(i,ipart)
            enddo
            ptemp(i)= ptemp(i) +prad(i)
         enddo
         shat1= ptemp(4)**2 -ptemp(3)**2 -ptemp(2)**2 -ptemp(1)**2
         pini1= sqrt( shat1 )/2.d0
         ppart(1,1)= 0.d0
         ppart(2,1)= 0.d0
         ppart(3,1)= pini1
         ppart(4,1)= pini1
         ppart(1,2)= 0.d0
         ppart(2,2)= 0.d0
         ppart(3,2)= -pini1
         ppart(4,2)= pini1
      endif

C  Soft-gluon correction.
      call sgcval( igsub, ppart, idpart, prad, wgt )
      weight= weight*wgt

      return
      end
C--------+---------+---------+---------+---------+---------+---------+
      subroutine rotmin( vector, refer, target )
C  Rotate "vector" so that "refer" should align along "target" with the 
C  same operation.
C  The rotation matrix is so determined that the change should be 
C  minimized when the difference between "refer" and "target" is small.
C  The use of this routine may be dangerous when the difference is large.
C  December 26, 2013, S.O.
      implicit real*8(a-h,o-z)
      dimension vector(3),refer(3),target(3),temp(3)
      dimension rotmtx(3,3),rotmtx1(3,3),rotmtx2(3,3)

C  Rotation matrix to align "refer" along the 3rd axis.
      vtrans= refer(1)**2 +refer(2)**2
      vabs= sqrt( vtrans +refer(3)**2 )
      if( vabs.lt.1.d-12 ) then
         print '('' rotmin: reference vector is too small.'')'
         stop
      endif
      vtrans= sqrt( vtrans )
      if( vtrans.lt.1.d-12 ) then
         csph= 1.d0
         snph= 0.d0
      else
         csph= refer(1)/vtrans
         snph= refer(2)/vtrans
      endif
      csth= refer(3)/vabs
      snth= sqrt( 1.d0 -csth**2 )
      rotmtx1(1,1)= csth*csph
      rotmtx1(1,2)= -snph
      rotmtx1(1,3)= snth*csph
      rotmtx1(2,1)= csth*snph
      rotmtx1(2,2)= csph
      rotmtx1(2,3)= snth*snph
      rotmtx1(3,1)= -snth
      rotmtx1(3,2)= 0.d0
      rotmtx1(3,3)= csth

C  Matrix to rotate a vector along the 3rd axis to the direction of "target".
C  The same azimuthal angle is used when it cannot be determined.
C  The matrix is exactly the inverse of the above matrix when the directions 
C  of "refer" and "target" are identical.
      vtrans= target(1)**2 +target(2)**2
      vabs= sqrt( vtrans +target(3)**2 )
      if( vabs.lt.1.d-12 ) then
         print '('' rotmin: target vector is too small.'')'
         stop
      endif
      vtrans= sqrt( vtrans )
      if( vtrans.gt.1.d-12 ) then
         csph= target(1)/vtrans
         snph= target(2)/vtrans
      endif
      csth= target(3)/vabs
      snth= sqrt( 1.d0 -csth**2 )
      rotmtx2(1,1)= csth*csph
      rotmtx2(2,1)= -snph
      rotmtx2(3,1)= snth*csph
      rotmtx2(1,2)= csth*snph
      rotmtx2(2,2)= csph
      rotmtx2(3,2)= snth*snph
      rotmtx2(1,3)= -snth
      rotmtx2(2,3)= 0.d0
      rotmtx2(3,3)= csth

C  Product of the two matrices.
      do i=1,3
         do j=1,3
            rotmtx(i,j)= 0.d0
            do k=1,3
               rotmtx(i,j)= rotmtx(i,j) +rotmtx2(i,k)*rotmtx1(k,j)
            enddo
         enddo
      enddo

C  Execute the rotation.
      do i=1,3
         temp(i)= 0.d0
         do k=1,3
            temp(i)= temp(i) +rotmtx(i,k)*vector(k)
         enddo
      enddo

      do i=1,3
         vector(i)= temp(i)
      enddo

      return
      end
